package org.apache.ws.muws.v1_0.impl;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.ws.muws.impl.CategoryImpl;
import org.apache.ws.muws.v1_0.MuwsConstants;
import org.apache.ws.muws.v1_0.capability.RelationshipsCapability;
import org.apache.ws.muws.v1_0.events.Situation;
import org.apache.ws.muws.v1_0.events.impl.SituationImpl;
import org.apache.ws.muws.v1_0.events.impl.XmlBeansManagementEvent;
import org.apache.ws.notification.base.NotificationProducerResource;
import org.apache.ws.notification.topics.Topic;
import org.apache.ws.notification.topics.TopicSpace;
import org.apache.ws.resource.properties.ResourcePropertyMetaData;
import org.apache.ws.resource.properties.impl.XmlBeansResourceProperty;
import org.apache.ws.resource.properties.impl.XmlBeansResourcePropertySet;
import org.apache.ws.util.XmlBeanUtils;
import org.apache.ws.util.i18n.Messages;
import org.apache.ws.util.i18n.MessagesImpl;
import org.apache.xmlbeans.XmlObject;
import org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart2.RelationshipCreatedNotificationDocument;
import org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart2.RelationshipDeletedNotificationDocument;
import org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart2.RelationshipType;
import org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart2.impl.RelationshipDocumentImpl;

import java.util.Iterator;


/**
 * A specialized ResourceProperty for generating RelationshipCreated and RelationshipDestroyed
 * events when a Add or Remove is called
 *
 * @author Sal Campana
 */
public class XmlBeansRelationshipResourceProperty extends XmlBeansResourceProperty
{
    private static final Log LOG = LogFactory.getLog( XmlBeansRelationshipResourceProperty.class );
    private static final Messages MSG = MessagesImpl.getInstance();
    private NotificationProducerResource m_resource;

    public XmlBeansRelationshipResourceProperty(ResourcePropertyMetaData resourcePropertyMetaData,
                                                XmlBeansResourcePropertySet xmlBeansResourcePropertySet,
                                                NotificationProducerResource resource)
    {
        super(resourcePropertyMetaData, xmlBeansResourcePropertySet);
        m_resource = resource;
    }

    /**
     * Overrides add() to generate RelationshipCreated notifications
     * @see XmlBeansResourceProperty#add(Object)
     */
    public void add(Object o)
    {
        super.add(o);
        throwRelationshipCreatedNotif((XmlObject)o);
    }

    /**
     * Overrides remove() to generate RelationshipDeleted notifications
     * @see XmlBeansResourceProperty#remove(Object)
     */
    public boolean remove(Object o)
    {
        XmlObject copiedRelationship = XmlBeanUtils.copyXmlBean((XmlObject)o);
        boolean wasRemoved = super.remove(o);
        if(wasRemoved)
        {
            throwRelationshipDeletedNotif(copiedRelationship);
        }
        return wasRemoved;
    }

    /**
     * Returns the RelationshipCreated Topic or null
     *
     * @return RelationshipCreated Topic or null
     */
    private Topic getCreatedTopic()
    {
        return getRelationshipsCapabilitySubTopic(RelationshipsCapability.SUBTOPIC_NAME_RELATIONSHIP_CREATED);
    }

    /**
     * Returns the RelationshipDeleted Topic or null
     *
     * @return RelationshipDeleted Topic or null
     */
    private Topic getDeletedTopic()
    {
        return getRelationshipsCapabilitySubTopic(RelationshipsCapability.SUBTOPIC_NAME_RELATIONSHIP_DELETED);
    }

    /**
     * Returns a given RelationshipsCapability-SubTopic
     *
     * @param subtopicName
     * @return The sub topic or null if undefined
     */
    private Topic getRelationshipsCapabilitySubTopic(String subtopicName)
    {
        Topic subTopic=null;
        TopicSpace topicSpace = m_resource.getTopicSpaceSet().getTopicSpace(MuwsConstants.NSURI_MUWS_PART2_TOPICS);
        if (topicSpace != null)
        {
            Topic relationshipCapTopic = topicSpace.getTopic(RelationshipsCapability.TOPIC_NAME);
            if(relationshipCapTopic != null)
            {
               subTopic = relationshipCapTopic.getTopic(subtopicName);
            }
        }
        return subTopic;
    }

    private void throwRelationshipCreatedNotif(XmlObject o)
    {
        Topic createdTopic = getCreatedTopic();
        if(createdTopic != null)
        {
           //if its there then build a notif and publish

            try
            {
                Situation situation = new SituationImpl( new CategoryImpl( MuwsConstants.SITUATION_CREATE ) );
                XmlBeansManagementEvent xme = new XmlBeansManagementEvent( situation );
                RelationshipCreatedNotificationDocument relationshipCreatedDoc = RelationshipCreatedNotificationDocument.Factory.newInstance();
                org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart2.RelationshipCreatedNotificationDocument.RelationshipCreatedNotification relationshipCreatedNotification = relationshipCreatedDoc.addNewRelationshipCreatedNotification();
                relationshipCreatedNotification.setRelationship(((RelationshipDocumentImpl)o).getRelationship());
                xme.setAny(new Object[]{relationshipCreatedDoc});
                //publish!
                createdTopic.publish(xme.getXmlObject());
            }
            catch (Exception e)
            {
                LOG.debug("Unable to throw RelationshipCreated notification. Cause:" + e.getLocalizedMessage(), e);
            }
        }
    }

    private void throwRelationshipDeletedNotif(XmlObject copiedRelationship)
    {
        Topic deletedTopic = getDeletedTopic();
        if(deletedTopic != null)
        {
            //if its there then build a notif and publish
            try
            {
                Situation situation = new SituationImpl( new CategoryImpl( MuwsConstants.SITUATION_DESTROY ) );
                XmlBeansManagementEvent xme = new XmlBeansManagementEvent( situation );
                RelationshipDeletedNotificationDocument relationshipDeletedDoc = RelationshipDeletedNotificationDocument.Factory.newInstance();
                org.oasisOpen.docs.wsdm.x2004.x12.muws.wsdmMuwsPart2.RelationshipDeletedNotificationDocument.RelationshipDeletedNotification relationshipDeletedNotification = relationshipDeletedDoc.addNewRelationshipDeletedNotification();
                relationshipDeletedNotification.setRelationship(((RelationshipDocumentImpl)copiedRelationship).getRelationship());
                xme.setAny(new Object[]{relationshipDeletedDoc});
                //publish!
                deletedTopic.publish(xme.getXmlObject());
            }
            catch (Exception e)
            {
                LOG.debug("Unable to throw RelationshipDeleted notification. Cause:" + e.getLocalizedMessage(), e);
            }
        }
    }

    /**
     * Overrides clear() to generate RelationshipDeleted notifications
     * @see XmlBeansResourceProperty#clear()
     */
    public synchronized void clear()
    {
        //get iterator and throw notif for each...do a copy
        Iterator iterator = iterator();
        while (iterator.hasNext())
        {
            Object relationship = iterator.next();
            throwRelationshipDeletedNotif(XmlBeanUtils.copyXmlBean((XmlObject)relationship));
        }
        super.clear();
    }
}
